import {connect} from "react-redux"

const tryParse = (string) => {
  try {
    return JSON.parse(string)
  } catch (error) {
    return null
  }
}

const BlockSettingsItem = connect(({blocks}, {id}) => ({block: blocks[id]}))(
  class BlockSettingsItem extends React.Component {
    state = {}

    static getDerivedStateFromProps(props, state) {
      if (state.block !== props.block) {
        return {
          block: props.block,
          contentString: JSON.stringify(props.block.content, null, 2),
          viewRefString: props.block.viewRef,
        }
      }
      return null
    }

    updateContent = (event) => {
      this.setState({contentString: event.target.value})
    }

    updateViewRef = (event) => {
      this.setState({viewRefString: event.target.value})
    }

    apply = () => {
      this.props.dispatch({
        type: "UPDATE_BLOCK",
        payload: {
          id: this.props.id,
          block: {content: tryParse(this.state.contentString), viewRef: this.state.viewRefString},
        },
      })
    }

    render() {
      return (
        <li className="blocks-settings-modal-item">
          <div>{this.props.id}</div>
          <div className="blocks-settings-modal-item-textarea-wrapper">
            <textarea
              className="blocks-settings-modal-item-textarea"
              value={this.state.contentString}
              onChange={this.updateContent}
            />
          </div>
          <div className="blocks-settings-modal-item-input-wrapper">
            <input
              className="blocks-settings-modal-item-input"
              value={this.state.viewRefString}
              onChange={this.updateViewRef}
            />
          </div>
          <div className="blocks-settings-modal-item-apply">
            <button className="button" onClick={this.apply}>
              Apply changes
            </button>
          </div>
        </li>
      )
    }
  },
)

const BlocksSettings = connect(({blocks}) => ({blocksIds: Object.keys(blocks)}))(
  class BlocksSettings extends React.Component {
    state = {show: false}

    show = () => {
      this.setState({
        show: true,
      })
    }

    hide = () => {
      this.setState({
        show: false,
      })
    }

    handleKeyDown = (event) => {
      if (this.state.show && event.keyCode == 27) {
        this.hide()
      }
    }

    componentDidMount() {
      document.addEventListener("keydown", this.handleKeyDown)
    }

    componentWillUnmount() {
      document.removeEventListener("keydown", this.handleKeyDown)
    }

    render() {
      return (
        <div className="blocks-settings">
          <button className="button" onClick={this.show}>
            Edit Blocks
          </button>
          {this.state.show && (
            <div className="blocks-settings-modal-wrapper">
              <div className="blocks-settings-modal-close">
                <button className="button" onClick={this.hide}>
                  x
                </button>
              </div>
              <ul className="blocks-settings-modal-list">
                {this.props.blocksIds.map((id) => (
                  <BlockSettingsItem key={id} id={id} />
                ))}
              </ul>
            </div>
          )}
        </div>
      )
    }
  },
)

export default BlocksSettings
