const _get = require("lodash/get")
const {auth: {read, write}} = require("./config")
const Api = require("./socket-api")
const url = require("url")

function checkHost(str) {
  const {protocol, host} = url.parse(str)
  if(host && ["http:", "https:"].includes(protocol)) return protocol+"//"+host
  throw "Invalid host"
}

const promptPassword = username => new Promise(resolve => {
  const prompt = require("prompt")
  prompt.message = prompt.delimiter = ""
  prompt.start({stdout: process.stderr})
  prompt.get([{
    name: "password",
    description: `Password for '${username}': `,
    hidden: true, required: true, replace: "*"
  }], (err, res) => {
    if(err){
      err.message == "canceled" || console.error(err)
      process.stderr.write("\n")
      process.exit(1)
    }
    resolve(res.password)
  })
})

const connect = host => Api(host, {
  timeout: 60 * 1000, keepConnection: 5 * 60 * 1000,
  socket: {secure: true, rejectUnauthorized: false, verify: false, transports: ['websocket', 'polling']}
})

async function login({_, username, password, host, tmp} = {}) {
  if(username && !_.includes("login")) {
    host = checkHost(host || "http://localhost:3000")
    const api = connect(host)
    api.host = host
    try{
      await api.auth.login({data: {username, password: password || await promptPassword(username)}})
    } catch(error) {
      console.error(error)
      process.exit(1)
    }
    return {api}
  }
  const config = read() || {}

  if(_.includes("login") && tmp) {
    host = checkHost(host)
    if(!username || !host) throw "Please login"
    const api = connect(host)
    api.host = host
    await api.auth.login({data: {username, password: password || await promptPassword(username)}})
    const {data} = await api.socket.refreshToken()
    config.tmp = config.tmp || {}
    config.tmp[process.ppid] = {username, token: data, host, time: Date.now()}
    write(config)
    return {api}
  }

  const _config = _get(config, ["tmp", process.ppid], config)
  username = username || _config.username
  host = checkHost(host || _config.host || "http://localhost:3000")

  if(!username || !host) throw "Please login"
  const api = connect(host)
  api.host = host

  if(_.includes("login") || _config.username != username || _config.host != host) {
    await api.auth.login({data: {username, password: password || await promptPassword(username)}})
    const {data} = await api.socket.refreshToken()
    Object.assign(_config, {username, token: data, host, time: Date.now()})
    write(config)
    return {api}
  }

  try {
    await api.auth.login({query: {refreshToken: _config.token}})
    const {data} = await api.socket.refreshToken()
    Object.assign(_config, {username, token: data, host, time: Date.now()})
    write(config)
    return {api}
  } catch (error) {
    if (error.code != 'api.auth.token_expired') {
      console.error(error)
      process.exit(1)
    }
    console.error("Token expired. Relogin")
  }

  try {
    await api.auth.login({data: {username, password: password || await promptPassword(username)}})
    const {data} = await api.socket.refreshToken()
    Object.assign(_config, {username, token: data, host, time: Date.now()})
    write(config)
    return {api}
  } catch (error) {
    console.error(error)
    process.exit(1)
  }
}

function logout() {
  const config = read() || {}
  if(_get(config, ["tmp", process.ppid])) {
    delete config.tmp[process.ppid]
    console.log("Temporary Logged out")
    return write(config)
  }
  return write({tmp: config.tmp})
}

login.cli = yargs => yargs.command({
  command: 'login',
  desc: 'Login to api',
  builder: yargs => yargs
    .demandOption('username')
    .demandOption('host')
    .option("tmp", {boolean: true, desc: "temporary login for terminal session"}),
  handler: argv => login(argv).then(({api}) => (console.log("Logged in"), api.close())).catch(error => {
    console.error(error)
    process.exit(1)
  })
}).command({
  command: 'logout',
  desc: 'Logout',
  handler() {return logout()},
})

module.exports = login
