const socketClient = require("socket.io-client")
const EventEmmitter = require("events")
const {version} = require("./package.json")

function attach(_socket, attachOpts = {}) {
  let socket
  let connectPromise
  let handleConnect = null
  let promises = {}
  let refreshToken = attachOpts.refreshToken
  let token = attachOpts.token
  const throwErrorOnStatus = attachOpts.throwErrorOnStatus == null ? true : !!attachOpts.throwErrorOnStatus
  const emitter = new EventEmmitter()

  async function callApi(body, opts = {}, __socket) {
    const socket = __socket || await handleConnect()

    const TIMEOUT = opts.timeout || attachOpts.timeout || 5000
    body = body || {}
    // body.action == "auth.login" && await callApi({action: "auth.logout"})
    // body.action == "auth.login" && (token = refreshToken = null)
    body.action == "auth.logout" && (token = refreshToken = null)
    ;["socket.setToken", "auth.login"].includes(body.action) && (body.socketioVersion = version)
    const messageId = callApi.id = (callApi.id || 0) + 1
    socket.emit("api", Object.assign(body, {messageId}))
    return await new Promise((resolve, reject) => {
      const timerId = setTimeout(() => {
        if(promises[messageId]) {
          delete promises[messageId]
          const error = opts.preparedError || new Error("Timeout expired")
          error.code = "TimeoutExpired"
          error.message = "Timeout expired"
          if(reject) {
            reject(error)
            resolve = reject = null
          }
        }
      }, TIMEOUT)
      promises[messageId] = Object.assign({}, {request: body}, opts, {
        resolve(...args) {
          delete promises[messageId]
          clearTimeout(timerId)
          if(resolve) {
            // console.log("resolve", messageId)
            resolve(...args)
            resolve = reject = null
          }
        },
        reject(...args) {
          delete promises[messageId]
          clearTimeout(timerId)
          if(reject) {
            // console.log("reject", messageId)
            reject(...args)
            resolve = reject = null
          }
        }
      })
    })
  }

  function close() {
    if(socket) {
      const _soc = socket
      socket = connectPromise = null
      _soc.close()
      const _promises = promises
      promises = {}

      for(const i in _promises) {
        const promise = _promises[i]
        delete _promises[i]
        const error = {
          message: "Connection is closed",
          name: "ConnectionClosed",
          code: "ECONCLOSED",
          request: promise.request,
        }
        promise.resolve({error, status: 503})
      }
      emitter.emit("closeConnection", _soc)
    }
  }

  const apiProxy = (path, hash = {}, preparedError) => {
    if(!preparedError) {
      preparedError = new Error("")
      Error.captureStackTrace && Error.captureStackTrace(preparedError, apiProxy)
    }
    path = path || []

    function get(target, prop) {
      if(typeof prop === "string" && prop !== "call" && prop !== "apply" && prop !== "inspect" && prop !== "valueOf" && prop !== "name") {
        if(!(prop in hash)) {
          if(!path.length && prop != "domain" && prop in emitter) {
            hash[prop] = (...args) => emitter[prop](...args)
          } else {
            const [sub, ...other] = prop.split(".")
            const preparedError = new Error("")
            Error.captureStackTrace && Error.captureStackTrace(preparedError, get)
            let proxy = hash[sub] = apiProxy([...path, sub], {}, preparedError)
            for(let i = 0; i < other.length; i++) proxy = proxy[other[i]]
            return proxy
          }
        }
        return hash[prop]
      }
    }

    return new Proxy(body => callApi(Object.assign({}, body, {action: path.join(".")}), {preparedError}), {
      get,
      set(target, prop, value) {
        hash[prop] = value
        return true
      },
      deleteProperty(target, prop) {
        delete hash[prop]
        return true
      }
    })
  }

  handleConnect = () => connectPromise = connectPromise || new Promise((resolve, reject) => {
    const socketOpts = Object.assign({transports: ["websocket", "polling"]}, attachOpts.socket)
    const sock = typeof _socket == "string" ? socketClient(_socket, socketOpts) : _socket

    sock.on("connect", async args => {
      if(refreshToken) {
        await callApi({action: "auth.login", query: {refreshToken}}, {}, sock).catch(error => {
          error.status == 401 && (refreshToken = token = null)
        })
        refreshToken && emitter.emit("login", {refreshToken})
      } else if(token) {
        await callApi({action: "socket.setToken", data: {token}}, {}, sock).catch(error => {
          error.status == 401 && (refreshToken = token = null)
        })
      }
      resolve && resolve(socket = sock)
      resolve = reject = null
      emitter.emit("connect", args)
    })
    sock.on("api", async data => {
      // console.log(">>", data, promises[data.messageId])
      try {
        if(data) {
          if(!data.messageId) {
            emitter.emit("notify", data)
          } else {
            if(!promises[data.messageId]) {
              /*eslint no-console: ["error", { allow: ["warn", "error"] }]*/
              console.warn("There is no handle for response ", data)
              return
            }
            // keepConnection()
            const promise = promises[data.messageId]
            try {
              promise.request.action == "auth.login" && data.data && (
                refreshToken = (await callApi({action: "socket.refreshToken"}, {}, sock)).data,
                token = (await callApi({action: "socket.token"}, {}, sock)).data
              )
            } catch(error) {
              console.error("unknown error", error)
              return promise.reject(error)
            }
            delete promises[data.messageId]
            data.request = promise.request
            if(!["auth.login", "auth.logout"].includes(promise.request.action) &&
              data.error && data.status == 401 && data.error.code == "api.auth.token_expired") {
              // console.log(data.error)
              const t = refreshToken
              token = refreshToken = null
              await callApi({action: "auth.login", query: {token: t}}, {}, sock).catch(() => {})
              if(token && refreshToken) {
                emitter.emit("login", {token, refreshToken})
                return await callApi(promise.request, {}, sock).then(promise.resolve).catch(promise.reject)
              }
            }
            if(data.error && throwErrorOnStatus) {
              promise.preparedError = Object.assign(promise.preparedError || new Error(""), data.error)
              promise.preparedError.status = promise.preparedError.status || data.status

              promise.preparedError.message = promise.preparedError.message || data.error.message || data.error.code
                || "" + data.error
              promise.preparedError.data = data
              promise.reject(promise.preparedError)
            } else {
              promise.resolve(data)
            }
          }
        }
      } catch(error) {
        console.error(error, error.stack)
      }
    })

    sock.on("error", error => {
      emitter.emit("error", error)
      reject && reject(error)
      resolve = reject = null
    })

    sock.on("reconnect_failed", () => {
      reject && reject(new Error("socket-io reconnect failed"))
      resolve = reject = null
    })
    sock.on("disconnect", arg => {
      emitter.emit("disconnect", arg)
      socket === sock && close()
    })
    sock.on("close", arg => {
      emitter.emit("closeConnection", arg)
    })
  })

  const preparedError = new Error("")
  Error.captureStackTrace && Error.captureStackTrace(preparedError, apiProxy)
  return apiProxy(null, {
    close,
    get refreshToken() {return refreshToken},
    get token() {return token},
    get rawSocket() {return socket},
    connect() {return handleConnect()}
  }, preparedError)
}

module.exports = attach
