const fetch = require("node-fetch");

const parseRef = ref => {
  if (!ref || !ref.match) return null;
  const matches = ref.match(
    /^gs:\/\/(\w[\w.-]+)\/((?:\w[\w.-]*)(?:\/\w[\w.-]*)*)$/
  );
  if (!matches) return null;
  const [, bucketName, path] = matches;
  return { bucketName, path };
};

const ref2url = ref => {
  const refo = parseRef(ref);
  const { path, bucketName } = refo;
  return bucketName.indexOf(".") >= 0
    ? `https://${bucketName}/${path}`
    : `https://storage.googleapis.com/${bucketName}/${path}`;
};

const getContent = async contentRef => {
  if (!contentRef) {
    return null;
  }
  const response = await fetch(ref2url(contentRef));
  if (response.status === 200) {
    const content = await response.json();
    return content;
  }

  return {};
};

const getBlockContent = block => {
  if (block.contentCache) {
    try {
      const content = JSON.parse(block.contentCache);
      return Promise.resolve(content);
    } catch (e) {
      console.log(e);
      return Promise.resolve(null);
    }
  }
  return block.contentRef
    ? getContent(block.contentRef)
    : Promise.resolve(null);
};

module.exports = getBlockContent;
