const _ = require("lodash/fp");
const getBlockContent = require("./get-block-content");
const prompt = require("./prompt");
const migrateContent = require("./migrate-content");

const LIMIT = 50;

const fetchBlocks = async (api, filter) => {
  const blocks = [];
  let skip = 0;
  for(;;) {
    const { data } = await api.block.list({
      query: { filter, limit: LIMIT, skip }
    });
    blocks.push(...data);
    if (data.length < LIMIT) {
      break;
    }
    skip += LIMIT;
  }
  return blocks;
};

const updateBlock = async (api, block, websiteId) => {
  if (_.equals(block.version, block.base.version)) {
    return;
  }
  await api.block.reroll({ id: block._id, source: block.base._id });
  if (!block.contentRef) {
    return;
  }
  const content = await getBlockContent(block);
  await api.block.update({
    id: block._id,
    data: { content: migrateContent(content, { strict: false }) },
    query: { payload: { identifier: 0, websiteId } }
  });
};

const migrate = async (api, { websiteId }) => {
  const websiteClone = await api.website.clone({ id: websiteId });
  const {
    data: { _id: websiteCloneId }
  } = websiteClone;
  console.log(`Website clone id: ${websiteCloneId}`)
  try {
    const blocks = await fetchBlocks(api, { website: websiteCloneId });
    for (const block of blocks) {
      await updateBlock(api, block, websiteCloneId);
    }
    await prompt("Press enter to remove clone");
  } finally {
    await api.website.remove({ id: websiteCloneId });
  }
};

module.exports = migrate;
