const _ = require("lodash");

const migrateCollection = type => source => {
  const arrayMap = _.get(collectionMap, type);

  console.log(type, arrayMap);
  if (!arrayMap) {
    console.warn(`Undefined collection type ${type}`);
    return source; // Can't migrate. Return as is.
  }

  const sourceArray = _.get(source, "items", source);

  const result = {
    items: _.map(sourceArray, item =>
      migration(item, arrayMap, { strict: true })
    )
  };

  return result;
};

function migrateImage(source) {
  return {
    type: "image",
    video: {},
    image: {
      src: source.src,
      alt: source.alt,
      resourceRef: source.resourceRef
    }
  };
}

function migrateText(source) {
  return { content: source };
}

const elementsMap = {
  // Texts
  title: ["title", migrateText],
  // subtitle: ["subtitle", migrateText],
  // heading: ["heading", migrateText],
  // subheading: ["subheading", migrateText],
  text: ["body", migrateText],
  // description: ["subtitle", migrateText],

  // Collections
  // articles: ["collection", migrateCollection("articles")],
  // testimonials: ["collection", migrateCollection("testimonials")],
  // collection: "collection",

  // // Buttons
  // button: "button",
  // cta: "button",
  // "button-1": "button",
  // "button-2": "button_additional",

  // background: "background"
};

const collectionMap = {
  articles: {
    title: ["item_heading", migrateText],
    description: ["item_body", migrateText],
    icon: "item_icon"
  },
  testimonials: {
    name: ["item_person", migrateText],
    position: ["item_category", migrateText],
    description: ["item_body", migrateText],
    date: ["item_date", migrateText],
    image: ["item_image", migrateImage]
  }
};

function migration(source, keyMap, options) {
  const result = {};
  for (const key in source) {
    if (source.hasOwnProperty(key)) {
      const newKey =
        (Array.isArray(keyMap[key]) ? keyMap[key][0] : keyMap[key]) || key;
      const transform = Array.isArray(keyMap[key])
        ? _.get(keyMap, `${key}[1]`, arg => arg)
        : arg => arg;
      console.log(key, newKey);

      if ((options.strict && keyMap[key]) || !options.strict) {
        result[newKey] = transform(source[key]);
      }
    }
  }
  return result;
}

module.exports = (source, options) => migration(source, elementsMap, options);
